'****************************************************************************
'GLTLIB.BAS Graphic library of subroutines for use with a Toshiba T6963C
'graphic LCD controller connected to a Microchip(TM) PIC 16F877 microcontroller
'Version 1.1
'Written in MELabs PicBasicPro(TM) Ver 2.40
'GLUTIL.EXE Ver 1.1 is an Windows based interface that is used with GLT6963C.BAS
'============================================================================
'
'The Graphic Library contains and initializes all the variables used.
'
'================== NOT FOR COMMERICIAL USE =================================
'
'       Author: Ranjit Diol, COMPSys Date: April, 2001
'            (c) Copyright COMPSys, April 2001
'
'   Last revision: March 2002
'
'   DISCLAIMER: This file is being released as non-commericial
'   software. It is being provided "AS IS", neither the author, 
'   nor COMPSys shall be held liable for any damages caused
'   by its use.
' 
'   This product is issued as a single license and can not
'   be distributed to others without the explicit consent of
'   COMPSys.
' 
'   SINGLE USER LICENSE: This application may be used in its entirety or 
'   in parts within some other non-commercial application(s) as long
'   as the those applications are used solely by the licensee.
'
'
'        Contact: rsdiol@compsys1.com for more information
'===============================================================================
'************************
' LCD Module constants  *
'************************
'
'Register set commands:
CURSOR_PTR_SET          CON     $21   'Cursor Pointer Set
OFFSET_REG_SET          CON     $22   'Offset Register Set
ADDR_PTR_SET            CON     $24   'Address Pointer Set

'Control Word Set commands:
TEXT_HOME_SET           CON     $40   'Text Home Address Set
TEXT_AREA_SET           CON     $41   'Text Area Set
GRAPH_HOME_SET          CON     $42   'Graphics Home address Set
GRAPH_AREA_SET          CON     $43   'Graphics Area Set

'Mode Set commands (OR with CG rom commands):
OR_MODE                 CON     $80   'OR mode
XOR_MODE                CON     $81   'XOR mode
AND_MODE                CON     $83   'AND mode
TEXT_ATTR_MODE          CON     $84   'Text Attribute mode
INT_CG_MODE             CON     $80   'Internal CG ROM mode
EXT_CG_MODE             CON     $88   'External CG ROM mode

'Display Mode commands (OR together required bits):
DISPLAY_OFF             CON     $90   'display off
BLINK_ON                CON     $91
CURSOR_ON               CON     $92
TEXT_ON                 CON     $94
GRAPHIC_ON              CON     $98
TEXT_AND_GRAPH_ON	CON     $9C

'Cursor Pattern Select
CURSOR_1LINE            CON     $A0
CURSOR_2LINE            CON     $A1
CURSOR_3LINE            CON     $A2
CURSOR_4LINE            CON     $A3
CURSOR_5LINE            CON     $A4
CURSOR_6LINE            CON     $A5
CURSOR_7LINE            CON     $A6
CURSOR_8LINE            CON     $A7

'Data Auto Read/Write
DATA_AUTO_WR            CON     $B0
DATA_AUTO_RD            CON     $B1
AUTO_DATA_RESET         CON     $B2

'Data Read/Write
DATA_WR_INC             CON     $C0   'Data write and increment addr
DATA_RD_INC             CON     $C1   'Data read and increment addr
DATA_WR_DEC             CON     $C2   'Data write and decrement addr
DATA_RD_DEC             CON     $C3   'Data read and decrement addr
DATA_WR                 CON     $C4   'Data write - no addr change
DATA_RD                 CON     $C5   'Data read - no addr change

'Screen Peek
SCREEN_PEEK             CON     $E0

'Screen Copy
SCREEN_COPY             CON     $E8

'Bit Set/Reset (OR with bit number 0-7)

BIT_RESET               CON     $F0
BIT_SET                 CON     $F8

'************************
' STATUS bits           *
'************************

STA0                    CON     0       'command excution status
STA1                    CON     1       'data read/write status
STA2                    CON     2       'auto data read status
STA3                    CON     3       'auto data write status
STA5                    CON     5       'controller status
STA6                    CON     6       'peek/copy error flag
STA7                    CON     7       'blink? or blank? status

ASCII_OFFSET		CON     $20     'ASCII offset for conversion
GL_PAUSE		CON     2
GL_ROW			CON     32
GL_COL			CON     64
GL_XMAX			CON     239
GL_XMIN			CON     0
GL_YMAX			CON     63
GL_YMIN                 CON     0
GL_LINES		CON	8


'Serial I/O definitions

out_pin	 VAR	PORTC.6		'Serial OUT pin
in_pin	 VAR    PORTC.7		'Serial IN pin
ser_baud CON	32		'16448 = 19200 Baud INVERTED, 32 = NON INV
in_dat   VAR    BYTE		'In coming byte data
in_ser   VAR    BYTE[15]        'In coming data buffer
in_lsb   VAR    BYTE            'In coming lsb of word
in_msb   VAR    BYTE            'In coming msb of word
gl_chr   VAR    BYTE            'Character byte
eep_addr VAR    WORD		'External eeprom address
b_inp1	 VAR    BYTE
b_inp2	 VAR    BYTE
b_inp3	 VAR    BYTE





'Variables used
' *** MODIFY AS REQUIRED FOR OTHER PINS AND PORTS *****
'
'Variables used
' *** MODIFY AS REQUIRED FOR OTHER PINS AND PORTS *****
'
glcd_tris 	VAR 	TRISD		'Data bus direction
glcd_dat	VAR 	PORTD		'LCD 8 bit data bus
glcd_wr	 	VAR 	PORTB.1		'Write control bit
glcd_rd	 	VAR 	PORTB.2		'Read control bit
glcd_ce	 	VAR 	PORTC.1		'Chip enable control bit
glcd_cd	 	VAR 	PORTC.2		'Command and Data toggle bit
glcd_rst	VAR 	PORTC.0		'Reset control bit
glcd_fsz 	VAR 	PORTC.5		'Font select control bit
glcd_bl	 	VAR 	PORTA.5		'Back light control bit
glcd_bsy	VAR 	PORTD.7         'In case there is a need, not really used
glcd_btn	VAR     PORTB.0		'A button for debugging purposes
glcd_sta0	VAR     PORTD.0		'Status Bit 0	
glcd_sta1	VAR     PORTD.1		'Status Bit 1
glcd_sta2   VAR     PORTD.2		'Status Bit 2
glcd_sta3   VAR     PORTD.3		'Status Bit 3

'I2C EEprom
scl	 VAR 	PORTC.3		'I2C SCL  for ext eeprom 
sda	 VAR	PORTC.4		'I2C SDA for ext eeprom
ctl  CON    $A0		'EEprom control code


'Set appropriate port/bit dir
TRISB.0  =      0
TRISB.1  =      0
TRISB.2  =      0
TRISD	 = 	0
TRISC.1	 =	0
TRISC.0	 =	0
TRISC.2	 =	0
TRISC.5	 =	0
TRISA.4	 =	0
TRISA.5	 =	0


'General use variables used
glcd_msb	VAR    	BYTE		'Data 1 variable
glcd_lsb	VAR    	BYTE 		'Data 2 variable
glcd_cmd	VAR    	BYTE 		'Command variable
gl_tmp	 	VAR    	BYTE		'Temp var		
gl_old	 	VAR     BYTE		'Previous byte var
gl_x     	VAR     WORD		'16 bit var for address loop
gl_y	 	VAR     WORD		'16 bit var for address loop
gl_i	 	VAR  	BYTE
gl_j	 	VAR     BYTE
gl_k	 	VAR     BYTE
gl_l	 	VAR     BYTE
gl_m	 	VAR     BYTE
num_1	 	VAR 	BYTE
num_2	 	VAR 	BYTE
gl_stp	 	VAR    	BYTE		'Step var in FOR..NEXT
gl_ad1	 	VAR    	BYTE		'Addr var
gl_ad2	 	VAR    	BYTE		'Addr var
graph_home 	VAR 	WORD		'LCD graphic start addr
graph_area 	VAR 	WORD		'LCD graphic bytes per line
text_home  	VAR 	WORD		'LCD text start addr
text_area  	VAR 	WORD		'LCD text chars per line
gl_xpos		VAR     BYTE
gl_ypos     VAR     BYTE
gl_addr     VAR     WORD
lcd_cmd		VAR     BYTE
rs_flag		VAR     BYTE		'Reset pixel flag
clr_flag    VAR     BYTE
fill_flag   VAR     BYTE		
gl_byte		VAR    	BYTE		'Graphic byte
gl_char		VAR     BYTE		'Text byte
g_char      VAR     BYTE
ptr_flag	VAR    	BYTE		'Addr ptr flag
gl_oldaddr	VAR  	WORD		'Var to keep track of address
gl_oldx		VAR     BYTE		'Variables to keep track
gl_oldy		VAR     BYTE		'of x and y values
glcd_fs		VAR     BYTE            'LCD font size
g_count     VAR    WORD
gl_flag		VAR    BYTE
ee_addr1    VAR    WORD
ee_addr2    VAR    WORD
ee_addr     VAR    WORD
glcd_fs = 0				'0=8x8 character space, 1=6x8

'Initialize variables
gl_tmp   =  0
gl_i     =  0
gl_stp   =  1
rs_flag  =  0
ptr_flag =  DATA_WR
gl_flag = 0



'Circle constants and variables
gl_pi			CON	2	'PI as an integer
glc_x			VAR     BYTE	'Calculation var
glc_y			VAR     BYTE	'Calculation var
glc_s			VAR     WORD	'Slope
glc_cx			VAR     BYTE	'Center x
glc_cy  		VAR     BYTE	'Center y
glc_r			VAR     BYTE	'Radius
g_num			VAR     WORD

'Line variables
gl_x1			VAR	BYTE
gl_x2			VAR	BYTE
gl_y1			VAR	BYTE
gl_y2			VAR	BYTE
gl_inc1			VAR	WORD
gl_inc2			VAR	WORD

'Chart Variables
s_xstart		VAR    BYTE
s_xend          VAR    BYTE
s_ystart        VAR    BYTE
s_yend          VAR    BYTE
s_tick          VAR    BYTE

'FS offsets
font_size		VAR     BYTE
font_byte		VAR     BYTE

'Terminal variables
in_array    VAR     BYTE[32]		'In coming array
in_ctl	    VAR     BYTE[8]			'In coming control array
buff_count	VAR  	BYTE		'General counter
gl_delay	VAR     BYTE		'Delay variable
gl_idx		VAR     BYTE		'General use
gl_idy		VAR     BYTE            'General use

'Erase Variables
X_Start VAR BYTE		'Start erase at x,y start and end at x,y end
Y_Start VAR BYTE		'These will need to be words on bigger screens
X_End   VAR BYTE
Y_End   VAR BYTE	 

'Initialize variables
gl_tmp   =  0
gl_i     =  0
rs_flag  =  0
clr_flag =  0
fill_flag = 0
ptr_flag =  DATA_WR


'Initial settings of port and bits
glcd_dat = 0	'Clear PORTD
glcd_wr  = 1    'Write high 
glcd_rd  = 1	'Read high
glcd_ce  = 1  	'Chip enable high (active low)
glcd_cd  = 1  	'Command high
glcd_fsz  = 0	'Set Font size (1=6x8 and 0=8x8 font)
glcd_rst = 1 	'RESET always high (low will reset the LCD controller)
Pause 500	'Let things stabilize

'Initialize LCD VRAM area variables
glcd_fs = 0

graph_home       	=     $1000    'Decimal 4096
text_home		=     $0000
IF glcd_fs = 0 Then
	font_size = 8
	font_byte = 7
	graph_area		=     $0020    '32 bytes per line
	text_area       =     $0020    '32 8x8 chars per line
   Else
	font_size = 6
	font_byte = 5
	graph_area		=    $0028    '40 bytes per line
	text_area      =     $0028    '40 6x8 chars per line
EndIF




glcd_rst = 0	' Reset after power-on
Pause 100
glcd_rst = 1

'*** jump past the routines below ***
GoTo continue_to_main 



'*********** PBP Subroutines used in this program *******

send_2:
	GoSub busy_chk
	glcd_dat = glcd_lsb
	glcd_cd  = 0
	glcd_rd  = 1
	glcd_ce  = 0
	glcd_wr  = 0
	'pauseus GL_PAUSE
	glcd_wr  = 1
	glcd_ce  = 1
send_1:
	GoSub busy_chk
	glcd_dat = glcd_msb
	glcd_cd  = 0
	glcd_rd  = 1
	glcd_ce  = 0
	glcd_wr  = 0
	'pauseus GL_PAUSE
	glcd_wr  = 1
	glcd_ce  = 1
send_cmd:
	GoSub busy_chk
	glcd_dat = glcd_cmd
	glcd_rd  = 1
	glcd_ce  = 0
	glcd_wr  = 0
	'pauseus GL_PAUSE
	glcd_wr  = 1
	glcd_ce  = 1
Return


busy_chk:
	glcd_ce = 1
	glcd_tris= %11111111
	glcd_cd = 1
	glcd_wr = 1
	glcd_ce = 0
	glcd_rd = 0
	'pauseus GL_PAUSE
	IF (glcd_sta0 <> 1) AND (glcd_sta1 <> 1) Then busy_chk
	glcd_rd = 1
        glcd_ce = 1
	glcd_tris = %00000000
Return

read1:
	GoSub busy_chk 'On return Ce =1. rd = 1, wr = 1, CD = 1
	glcd_tris = %11111111
        glcd_cd  = 0
        glcd_rd  = 0
	glcd_ce  = 0
	PauseUs GL_PAUSE
	gl_chr = glcd_dat
        glcd_ce = 1
	glcd_rd = 1
	glcd_tris = %00000000
Return

read_lcd:
'Assumes gl_x and gl_y and ptr_flag are given. Returns gl_chr
	GoSub gl_gaddr
	'glcd_cmd = ptr_flag
	'gosub send_cmd
	GoSub read1
Return

gl_resetdot:
	rs_flag = 1
gl_setdot:		'Set a pixel given gl_x,gl_y
	gl_addr	= graph_home + (gl_y * GL_ROW) + (gl_x/font_size)
	gl_oldaddr = gl_addr
	gl_oldx = gl_x
	gl_oldy = gl_y
	glcd_msb = gl_addr.BYTE1
	glcd_lsb = gl_addr.BYTE0
	glcd_cmd = ADDR_PTR_SET
	GoSub send_2

	'Check for reset flag
	IF rs_flag = 0 Then
		glcd_cmd = ($f8 | (font_byte - (gl_x//font_size)))
	   Else
		glcd_cmd = ($f0 | (font_byte - (gl_x//font_size)))
	EndIF
	GoSub send_cmd
        rs_flag = 0 'Reset flag
Return

'Set address given x,y coordinates
gl_gaddr:
	gl_addr	= graph_home + (gl_y * GL_ROW) + (gl_x/font_size)
	gl_oldaddr = gl_addr
	glcd_msb = gl_addr.BYTE1
	glcd_lsb = gl_addr.BYTE0
	glcd_cmd = ADDR_PTR_SET
	GoSub send_2
Return


set8:
	gl_x = glc_x+glc_cx : gl_y=glc_y+glc_cy	        '1
	GoSub gl_setdot
	gl_x = glc_cx-glc_x : gl_y=glc_y+glc_cy		'2	
	GoSub gl_setdot
	gl_x = glc_cx+glc_x : gl_y=glc_cy-glc_y		'3	
	GoSub gl_setdot
	gl_x = glc_cx-glc_x : gl_y=glc_cy-glc_y		'4	
	GoSub gl_setdot
	gl_x = glc_y+glc_cx : gl_y=glc_x+glc_cy		'5
	GoSub gl_setdot			
	gl_x = glc_cx-glc_y : gl_y=glc_cy+glc_x		'6	
	GoSub gl_setdot
	gl_x = glc_cx+glc_y : gl_y=glc_cy-glc_x		'7	
	GoSub gl_setdot
	gl_x = glc_cx-glc_y : gl_y=glc_cy-glc_x		'8	
	GoSub gl_setdot
Return


'Initialize and clear display
gl_init:				'Initialization

	'See data sheets for more info for the commands and parameters
        
	' Set Graphic Home Addr
	glcd_msb = graph_home.BYTE1		
	glcd_lsb = graph_home.BYTE0
	glcd_cmd = GRAPH_HOME_SET
        GoSub send_2

	' Set Graphic Area
	glcd_msb = graph_area.BYTE1
	glcd_lsb = graph_area.BYTE0
	glcd_cmd = GRAPH_AREA_SET
        GoSub send_2

        ' Set Text Home Addr
	glcd_msb = text_home.BYTE1
	glcd_lsb = text_home.BYTE0
	glcd_cmd = TEXT_HOME_SET
        GoSub send_2

	' Set Text Area
	glcd_msb = text_area.BYTE1
	glcd_lsb = text_area.BYTE0
	glcd_cmd = TEXT_AREA_SET
        GoSub send_2

	' Set OR mode
	glcd_cmd = OR_MODE
	GoSub send_cmd

grf_clr:
	' Set Addr Ptr
	glcd_msb = $00
	glcd_lsb = $00
	glcd_cmd = ADDR_PTR_SET
	GoSub send_2

	' Set Graphic mode on	
	glcd_cmd = GRAPHIC_ON
	GoSub send_cmd

    For gl_i = 1 to 240			'Clear Graphic Screen of a 240x64 screen
	For gl_j = 1 to 64
	  glcd_msb = $00			'Clear byte
	  glcd_cmd = DATA_WR_INC	'Command
	  GoSub send_1				'Send glcd_msb followed by glcd_cmd
	Next
    Next						'Do it 240x64 times
	' Set Graphic mode on	
	glcd_cmd = TEXT_AND_GRAPH_ON
	GoSub send_cmd
Return


'End of Init routine

'*** gl_circle d = diameter r = radius x,y are point locations ***
'***** Given Center = glc_cx,glc_cy and Radius = glc_r **********
gl_circ:
  glc_x = 0
  glc_y = glc_r
  glc_s = gl_pi-2*glc_r
  

  While (glc_x <= glc_y) 
 
      GoSub set8

	IF (glc_s.15 =1) Then
           glc_s = glc_s + (4*glc_x + 6)
         Else
           glc_s = glc_s + (4*(glc_x-glc_y) + 10)
           glc_y = glc_y - 1
	EndIF

	glc_x = glc_x + 1
  Wend

Return

'*** 4 sided Star ***
'***** Given Center = gl_cx,gl_cy and Radius = gl_r **********
gl_star4:
  glc_x = 0
  glc_y = glc_r
  glc_s = gl_pi-2*glc_r
  While (glc_x <= glc_y) 
     GoSub set8
     glc_s = glc_s + (4*(glc_x-glc_y) + 10)
     glc_y = glc_y - 3
     glc_x = glc_x + 1
  Wend
Return

'Line subroutines
gl_line:

GoSub chk_xy		'Make sure x1,x2,y1,y2 values are correct

gl_l = gl_x2 - gl_x1
gl_m = gl_y2 - gl_y1

IF gl_m > gl_l Then gl_line2		'If Y delta > X delta

'Assumes X delta > Y delta

glc_s = ((gl_m<<2)-gl_l)
gl_y = gl_y1
gl_inc1 = gl_m << 2
gl_inc2 = 2*(gl_m - gl_l)

For gl_x = gl_x1 to gl_x2
   IF glc_s.15 = 1 Then
     glc_s = glc_s + gl_inc1
    Else
     glc_s = glc_s + gl_inc2
     gl_y = gl_y+1
   EndIF
   GoSub gl_setdot
Next
Return

gl_line2:

'Assumes Y delta > X delta

glc_s = ((gl_l<<2) - gl_m)
gl_x = gl_x1
gl_inc1 = gl_l<<2
gl_inc2 = 2*(gl_l - gl_m)

For gl_y = gl_y1 to gl_y2
   IF glc_s.15 = 1 Then
     glc_s = glc_s + gl_inc1
    Else
     glc_s = glc_s + gl_inc2
     gl_x = gl_x+1
   EndIF
   GoSub gl_setdot
Next
Return
'This routine checks if x1 > x2 and if y1 > y2 if so it swaps them
chk_xy:
	IF gl_x1 > gl_x2 Then
		Swap gl_x1,gl_x2
                IF gl_x2 > GL_XMAX Then gl_x2 = GL_XMAX
        EndIF
	IF gl_y1 > gl_y2 Then
		Swap gl_y1,gl_y2
                IF gl_y2 > GL_YMAX Then gl_y2 = GL_YMAX
        EndIF
        
Return

gl_txt_pos:
	gl_addr = (text_home + (gl_y/8 * GL_ROW) + gl_x/font_size)
	glcd_msb = gl_addr.BYTE1
	glcd_lsb = gl_addr.BYTE0
	glcd_cmd = ADDR_PTR_SET
	GoSub send_2
Return



put_char:			'Assumes g_char,gl_x,gl_y and ptr_flag
	GoSub txt_home
        GoSub send_2
	GoSub txt_on
	gl_char = g_char - ASCII_OFFSET
put_it:
	GoSub gl_txt_pos	'Set text addr given gl_x,gl_y
	glcd_msb = gl_char
	glcd_cmd = ptr_flag
	GoSub send_1
	GoSub grf_txt_on
Return

'Place a digit as ascii on the LCD
put_5dig:
	'Assumes g_num,gl_x,gl_y and ptr_flag
	g_char = g_num DIG 4
	GoSub lookup_dig
	gl_x = gl_x + 8
put_4dig:
	g_char = g_num DIG 3
	GoSub lookup_dig
	gl_x = gl_x + 8
put_3dig:
	g_char = g_num DIG 2
	GoSub lookup_dig
	gl_x = gl_x + 8
put_2dig:
	g_char = g_num DIG 1
	GoSub lookup_dig
	gl_x = gl_x + 8	
put_1dig:
	g_char = g_num DIG 0
	GoSub lookup_dig
Return

lookup_dig:
         LookUp g_char,["0123456789"],gl_tmp
	 gl_char = gl_tmp - $20
         GoSub put_it
Return

put_byte:		      'Assumes gl_x,gl_y and gl_byte and ptr_flag
	GoSub txt_home
	GoSub txt_on         'Graphic and Text mode
	GoSub gl_gaddr        'Set address
	glcd_msb = gl_byte
	glcd_cmd = ptr_flag
	GoSub send_1
	GoSub grf_txt_on
Return



grf_on:

        GoSub send_2
	glcd_cmd = GRAPHIC_ON
	GoSub send_cmd
Return

txt_on:
	GoSub txt_home
	glcd_cmd = TEXT_ON
	GoSub send_cmd
Return

grf_txt_on:
	GoSub txt_home
	glcd_cmd = TEXT_AND_GRAPH_ON
	GoSub send_cmd
Return


'***** Convert incoming ascii to hex byte Routine *****
gl_find:
LookDown in_ser[gl_i],["0123456789abcdef"],in_dat
Return	

grf_home:	
	' Set Addr Ptr to graphic home
	glcd_msb = graph_home.BYTE1		
	glcd_lsb = graph_home.BYTE0
	glcd_cmd = GRAPH_HOME_SET
        GoSub send_2
Return

txt_home:	

        ' Set Text Home Addr
	glcd_msb = text_home.BYTE1
	glcd_lsb = text_home.BYTE0
	glcd_cmd = TEXT_HOME_SET
        GoSub send_2

	' Set Text Area
	'glcd_msb = text_area.BYTE1
	'glcd_lsb = text_area.BYTE0
	'glcd_cmd = TEXT_AREA_SET
        'gosub send_2

	' Set OR mode
	'glcd_cmd = OR_MODE
	'gosub send_cmd
Return

'Scale subroutines which draws an XY scale with tick marks
'Assumes values assigned to s_xtart, s_xend, s_ystart, s_yend, and s_tick
'and also that the addr ptr is set before drawing each axis
'Uses variables gl_x,gl_y and the gl_setdot subroutine
gl_yaxis:
	'Draw vertical Y axis
	'Set addr ptr

	gl_x = s_xstart
	For gl_y = s_ystart to s_yend
	GoSub gl_setdot
	Next

	'Add the tick marks to the Y axis
	gl_x = s_xstart+1
	For gl_y = s_ystart to s_yend step s_tick
	GoSub gl_setdot
	Next
Return

gl_xaxis:

	'Draw horizontal X axis
	gl_y = s_yend
	For gl_x = s_xstart to s_xend
	GoSub gl_setdot
	Next
	
	'Draw X axis tick marks
	gl_y = s_yend - 1
	For gl_x = s_xstart to s_xend step s_tick
	GoSub gl_setdot
	Next
Return

'end of scale routines

'Bar subroutine given width, height and x,y locatation
gl_bar:
        GoSub chk_xy

	For gl_i= gl_x1 to gl_x2 step s_tick
	   For gl_j= gl_y1 to gl_y2 step s_tick
		 gl_x = gl_i
		 gl_y = gl_j
		GoSub gl_setdot
	  Next gl_j
        Next gl_i

Return

'Rectangle subroutine given gl_x1,gl_x2,gl_y1, gl_y2
gl_rect:

        GoSub chk_xy         
	For gl_i= gl_x1 to gl_x2 step s_tick
		gl_x = gl_i
		gl_y = gl_y1
		GoSub gl_setdot
	        gl_y = gl_y2
		GoSub gl_setdot
	Next gl_i

	For gl_i= gl_y1 to gl_y2 step s_tick
		gl_y = gl_i
		gl_x = gl_x1
		GoSub gl_setdot
		gl_x = gl_x2
		GoSub gl_setdot
	Next gl_i

Return

'*** gl_gl_square45 turned 45 degrees ***
'***** Given Center = gl_cx,gl_cy and Radius = gl_r **********
gl_square45:
  glc_x = 0
  glc_y = glc_r
  glc_s = gl_pi-2*glc_r
  While (glc_x <= glc_y) 
     GoSub set8
     glc_s = glc_s + (4*(glc_x-glc_y) + 10)
     glc_y = glc_y - 1
     glc_x = glc_x + 1
  Wend
Return

'*** gl_square  ***
'***** Given Center = gl_cx,gl_cy and Radius = gl_r **********
gl_square:
  glc_x = 0
  glc_y = glc_r
  glc_s = gl_pi-2*glc_r
  While (glc_x <= glc_y) 
     GoSub set8
     glc_s = glc_s + (4*glc_x + 6)
     glc_x = glc_x + 1
  Wend
Return


'**** Following Code is used primarily for use with glt6963c.bas ********
'**** However, they can be adapted for use in other applications ********

'Subroutines to get serial data

send_one:
    	  SerOut2 out_pin,ser_baud,[HEX2 gl_chr]     'Send gl_chr out to I/O   
Return

get_one:
	SerIn2 in_pin,ser_baud,[wait("]"),STR in_ser\2]
	For gl_i = 0 to 1	'Convert to hex nibbles
	GoSub gl_find
	in_ser[gl_i]=in_dat
	Next gl_i
	'Now convert to hex bytes
	b_inp1 = ((in_ser[0]<<4) | (in_ser[1]))
Return

get_two:
	SerIn2 in_pin,ser_baud,[wait("]"),STR in_ser\4]
	For gl_i = 0 to 3	'Convert to hex nibbles
	GoSub gl_find
	in_ser[gl_i]=in_dat
	Next gl_i
	'Now convert to hex bytes
	b_inp1 = ((in_ser[0]<<4) | (in_ser[1]))
	b_inp2 = ((in_ser[2]<<4) | (in_ser[3]))
Return

get_three:
	SerIn2 in_pin,ser_baud,[wait("]"),STR in_ser\6]   'circle = ]c]40201e
 
	For gl_i = 0 to 5			 	  'Convert to hex nibbles
	   GoSub gl_find
	   in_ser[gl_i]=in_dat
	Next gl_i

	'Now convert to hex bytes
	b_inp1  = ((in_ser[0]<<4) | (in_ser[1]))
	b_inp2  = ((in_ser[2]<<4) | (in_ser[3]))
   	b_inp3  = ((in_ser[4]<<4) | (in_ser[5]))
	
Return

get_tick:
       SerIn2 in_pin,ser_baud,[wait ("]"),in_dat]    'Wait for ] and the load in_dat
       s_tick = in_dat - $30                         'Strip ASCII value
       gl_chr = in_dat				     'Load gl_chr with the char	
Return

get_four:
	SerIn2 in_pin,ser_baud,[wait("]"),STR in_ser\8]
	For gl_i = 0 to 7	'Convert to hex nibbles
	GoSub gl_find
	in_ser[gl_i]=in_dat
	Next gl_i
	'Now convert to hex bytes
	gl_x1  = ((in_ser[0]<<4) | (in_ser[1]))
	gl_x2  = ((in_ser[2]<<4) | (in_ser[3]))
	gl_y1  = ((in_ser[4]<<4) | (in_ser[5]))
	gl_y2  = ((in_ser[6]<<4) | (in_ser[7]))

Return



'A subroutine to erase a rectangle of screen of X_Start,Y_Start to X_End,Y_End
grf_clr_Rect:     
    gosub chk_xy
    For gl_x = gl_x1 to gl_x2				
	For gl_y = gl_y1 to gl_y2
	  if clr_flag = 1 then rs_flag = 1
  	  GoSub gl_setdot       'Set the dot (pixel)
	  rs_flag = 0	 	
	Next
    Next    
Return



'NOTE: These subroutines used by glt6963c.bas only
c_assign:
   glc_cx = b_inp1		'Assign them
   glc_cy = b_inp2
   glc_r  = b_inp3
Return

xy_assign:
   gl_x = b_inp1
   gl_y = b_inp2
Return





'**********  End of PBP subroutines *********************
'========================================================
'************End of all subroutines *********************


'********************************************************                                                     
'************* Jump to INIT section ********************* 
'********************************************************   
'Entry point to the main program which can call the above subroutines as needed
continue_to_main:



